use axum::{
    body::Body,
    extract::State,
    http::{header, Response, StatusCode},
    response::IntoResponse,
};
use axum_controller::*;
use hypertext::{maud, GlobalAttributes, Renderable};
use rust_embed::RustEmbed;

use crate::{ui::html_elements, AppState};

pub struct AxumEmbedAsset<T>(pub T);

impl<T> IntoResponse for AxumEmbedAsset<T>
where
    T: Into<String>,
{
    fn into_response(self) -> Response<Body> {
        let path = self.0.into();
        #[derive(RustEmbed)]
        #[folder = "public/"]
        struct EmbedAsset;

        fn markup_404(uri: String) -> impl Renderable {
            maud! {
                h1 { "404" }
                p { (uri) " Not Found" }
                @for i in 0..5 {
                    div .{"m-" (i)} { (i) }
                }
            }
        }

        match EmbedAsset::get(path.as_str()) {
            Some(content) => {
                let mime = mime_guess::from_path(path).first_or_octet_stream();

                ([(header::CONTENT_TYPE, mime.as_ref())], content.data).into_response()
            }
            None => (
                StatusCode::NOT_FOUND,
                axum::response::Html(markup_404(path).render()),
            )
                .into_response(),
        }
    }
}

pub struct AssetsController;

#[controller(state=AppState, path="/dist")]
impl AssetsController {
    #[route(GET "/*path")]
    #[tracing::instrument]
    async fn static_handler(path: String, _: State<AppState>) -> impl IntoResponse {
        let path = path.trim_start_matches('/').to_string();

        AxumEmbedAsset(path)
    }
}
