use serde::{Deserialize, Serialize};
use uuid::Uuid;

mod backend_settings;
mod chat;
// pub mod backend_process;
// pub mod backend_settings;
// pub mod chat;

pub use backend_settings::*;
pub use chat::*;

#[derive(Clone, Debug, Serialize, Deserialize, Hash, PartialEq)]
pub enum ChatRole {
    System,
    User,
    Assistant,
}

impl From<ChatRole> for String {
    fn from(val: ChatRole) -> Self {
        match val {
            ChatRole::System => "system",
            ChatRole::User => "user",
            ChatRole::Assistant => "assistand",
        }
        .to_string()
    }
}

#[derive(Clone, Debug, Serialize, Deserialize, Hash, PartialEq)]
pub struct ChatMessage {
    pub id: Uuid,
    pub role: ChatRole,
    pub content: String,
    _meta: String,
}

impl ChatMessage {
    pub fn new(role: ChatRole, content: String) -> Self {
        Self {
            id: Uuid::now_v7(),
            role,
            content,
            _meta: "".to_string(),
        }
    }
}

#[derive(Clone, Debug, Serialize, Deserialize, Hash, PartialEq)]
pub enum ChannelMessage {
    Token(String),
    Stop,
}

#[derive(Clone, Debug, Serialize, Deserialize)]
pub struct Chat {
    // sampler_settings: SamplerSettings,
    pub id: Uuid,
    pub history: Vec<ChatMessage>,
    _meta: String,
}

impl Chat {
    pub fn new(id: Uuid, history: Vec<ChatMessage>, meta: String) -> Self {
        Self {
            id,
            history,
            _meta: meta,
        }
    }
}
